<?php

namespace Drp\SimpleCms\Http\Controllers\Admin;

use Drp\SimpleCms\Contracts\ContentFactory;
use Drp\SimpleCms\Contracts\ContentManager;
use Drp\SimpleCms\Contracts\ContentRequest;
use Drp\SimpleCms\Exceptions\NoUpdatesMade;
use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Routing\Controller;

/**
 * Class ManageContentController
 * @package Drp\SimpleCms
 */
class ManageContentController extends Controller
{

    use AuthorizesRequests;

    /**
     * @var ContentFactory
     */
    private $contentFactory;
    /**
     * @var ContentManager
     */
    private $contentManager;


    /**
     * ContentController constructor.
     * @param ContentFactory $contentFactory
     * @param ContentManager $contentManager
     */
    public function __construct(
        ContentFactory $contentFactory,
        ContentManager $contentManager
    ) {
        $this->contentFactory = $contentFactory;
        $this->contentManager = $contentManager;
    }

    /**
     * Show the form for editing the specified content.
     *
     * @param string $contentType The type of content (see ContentFactory)
     * @param integer|null $contentId
     * @return \Illuminate\Http\Response
     */
    public function index($contentType, $contentId = null)
    {
        $content = $this->contentFactory->latestVersionByTypeAndContentId($contentType, $contentId);

        return view('simpleCms::manage', compact('content', 'contentType'));
    }

    /**
     * Store a newly created Content in storage.
     *
     * @param string $contentType
     * @return \Illuminate\Http\Response
     */
    public function store($contentType)
    {
        $content = $this->contentManager->createContent($contentType, request()->input());
        flash()->success(trans('simpleCms::admin.created', [
            'label' => ucfirst(content_label($contentType, false))
        ]));

        return redirect($content->adminEditUrl());
    }

    /**
     * Update the specified content in storage.
     *
     * @param ContentRequest $request
     * @param string $contentType The type of content (see ContentFactory)
     * @param integer $contentId
     * @return \Illuminate\Http\Response
     * @SuppressWarnings("unused")
     */
    public function update(ContentRequest $request, $contentType, $contentId)
    {
        $content = $this->contentFactory->latestVersionByTypeAndContentId($contentType, $contentId);

        try {
            $this->contentManager->updateContent($content, $request->prepareInput($content));
            flash()->success(trans('simpleCms::admin.updated', [
                'label' => ucfirst(content_label($contentType, false))
            ]));
        } catch (NoUpdatesMade $e) {
            flash()->info(trans('simpleCms::content.no-change'));
        }

        return redirect($content->adminEditUrl());
    }

    /**
     * Delete the specified content.
     *
     * @param string $contentType
     * @param integer $contentId
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy($contentType, $contentId)
    {
        $content = $this->contentFactory->latestVersionByTypeAndContentId($contentType, $contentId);

        $this->authorize('delete', $content);
        $content->destroyAllVersion();

        flash()->success(trans('simpleCms::admin.deleted', ['label' => 'Content']));

        return redirect()->route('admin.content.index', [
            'type' => $contentType,
        ]);
    }
}
