<?php

namespace Drp\SimpleCms;

use Drp\ProjectHelpers\ProjectHelpersServiceProvider;
use Drp\SimpleCms\Contracts\ContentFactory as ContentFactoryContract;
use Drp\SimpleCms\Contracts\ContentManager as ContentManagerContract;
use Drp\SimpleCms\Contracts\ContentRequest as ContentRequestContract;
use Drp\SimpleCms\Contracts\Publishable;
use Drp\SimpleCms\Generators\Commands\ContentMakeCommand;
use Drp\SimpleCms\Generators\Commands\ModelMakeCommand;
use Drp\SimpleCms\Http\Middleware\PreviewMode;
use Drp\SimpleCms\Http\Middleware\Published;
use Drp\SimpleCms\Http\Requests\ContentRequest;
use Drp\SimpleCms\Policies\PublishablePolicy;
use Drp\SimpleCms\ViewComposers\PreviewModeComposer;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;

/**
 * Class SimpleCmsServiceProvider
 * @package Drp\SimpleCms
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class SimpleCmsServiceProvider extends ServiceProvider
{
    /**
     * Perform post-registration booting of services.
     *
     * @return void
     */
    public function boot()
    {
        $this->loadRoutesFrom(__DIR__ . '/../routes/admin.php');
        $this->loadViewsFrom(__DIR__ . "/../resources/views", 'simpleCms');
        $this->loadTranslationsFrom(__DIR__ . "/../resources/lang", 'simpleCms');

        $this->publishes([
            __DIR__ . "/../resources/views" => base_path('resources/views/vendor/simpleCms'),
        ], 'views');

        $this->publishes([
            __DIR__ . '/../config/simple_cms.php' => config_path('simple_cms.php'),
        ], 'config');

        $this->registerPolicies();
        $this->registerViewComposers();
        $this->registerMiddleware();
    }

    /**
     * Register bindings in the container.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfigFrom(__DIR__ . '/../config/simple_cms.php', 'simple_cms');

        $this->app->register(ProjectHelpersServiceProvider::class);

        $this->app->bind(ContentRequestContract::class, ContentRequest::class);
        $this->app->bind(ContentFactoryContract::class, ContentFactory::class);
        $this->app->bind(ContentManagerContract::class, ContentManager::class);

        $this->registerGenerator();
    }


    /**
     * Register the application's policies.
     *
     * @return void
     */
    public function registerPolicies()
    {
        Gate::policy(Publishable::class, PublishablePolicy::class);
    }

    /**
     * Register the packages view composers to the relevant views.
     * @return void
     */
    protected function registerViewComposers()
    {
        view()->composer(
            [
                'simpleCms::partials.preview-bar',
                'simpleCms::partials.preview-link',
            ],
            PreviewModeComposer::class
        );
    }

    /**
     * Register the packages middleware.
     * @return void
     */
    protected function registerMiddleware()
    {
        Route::aliasMiddleware('previewMode', PreviewMode::class);
        Route::aliasMiddleware('published', Published::class);
    }

    /**
     * Register the generator commands
     * @return void
     */
    protected function registerGenerator()
    {
        $this->app->singleton('command.content', function ($app) {
            return $app[ContentMakeCommand::class];
        });
        $this->commands('command.content');

        $this->app->singleton('command.content-model', function ($app) {
            return $app[ModelMakeCommand::class];
        });
        $this->commands('command.content-model');
    }
}
